;---------------------------------------------------------------------------------------------
; PMDKeyface.asm
;
;  Created: 21.9.2014 12:54:19
;  Author: Zdenek Sestak
;
;---------------------------------------------------------------------------------------------
#include "tn2313def.inc"
;
; Hardwarov konfigurace:
;  - taktovno internm RC osciltorem 8MHz
;  - SCK, MISO, MOSI - voln pro pipojen programtoru
;  - RXD/XCK - pipojena PS/2 klvesnice (ponechno v mdu Keyboard Scan Codes: Set 2)
;  
;
;                                         AtTiny 2313
;                                        .-----__-----.         
;                   AVR RESET >----- PA2 |            | VCC ----< +5V
;                                        |            |           
;                        RXD* >----- PD0 |            | PB7 ----- SCK   
;                                        |            |           
;                   PMD RESET >----- PD1 |            | PB6 ----- MISO   
;                                        |            |             
;                    PMD STOP >----- PA1 |            | PB5 ----- MOSI
;                                        |            |             
;                   PMD SHIFT >----- PA0 |            | PB4 ----< 8255 PB4
;                                        |            |           
;                        XCK* >----- PD2 |            | PB3 ----< 8255 PB3 
;                                        |            |           
;                   8255 PA0  >----- PD3 |            | PB2 ----< 8255 PB2
;                                        |            |           
;                   8255 PA1  >----- PD4 |            | PB1 ----< 8255 PB1
;                                        |            |           
;                   8255 PA2  >----- PD5 |            | PB0 ----< 8255 PB0
;                                        |            |            
;                             >----- GND |            | PD6 ----< 8255 PA3
;                                        `------------'													  |
;
; * nutno pipojit extern pull-up rezistory, viz pipojen PS/2 klvesnice
;---------------------------------------------------------------------------------------------
; Konstanty v programu:
; r18 = 0x1f 
; r19 = 0x00
;---------------------------------------------------------------------------------------------
#define	_0x1f	r18
#define	_0x00	r19
;---------------------------------------------------------------------------------------------
; RAM 
; 0x00 .. 0x0F ~ odpovd registrm procesoru r0 a r15 - aktuln stavy sloupc
;---------------------------------------------------------------------------------------------
	.dseg
;---------------------------------------------------------------------------------------------
; RESET vector
;---------------------------------------------------------------------------------------------
	.cseg
reset:
	; nastaven konstant
	ldi		_0x1f, 0x1f
	ldi		_0x00, 0x00
	; pznak T se pouv jako indikace uvolnn klvesy (pijet 0xF0)
	clt
	; PORTB a PORTA - pull-up (stav nestisknutch klves)
	out		DDRB,  _0x00
	out		DDRA,  _0x00
	out		PORTB, _0x1f
	out		PORTA, _0x1f
	; PORTD - vstup
	out		DDRD, _0x00
	out		PORTD, _0x00
	; nulovn registr r0 - r15
	ldi		XH, 0
	ldi		XL, 0
	ldi		r17, 16
clear:
	st		X+, _0x00
	dec		r17
	brne	clear
	; Z - ukazatel na pevodn tabulku
	ldi		ZH, HIGH(2*table1)
	; konfigurace USARTU
	; - povolen pjmu, 8 bitov znak, lich parita
	; - synchronn md (clock = extern XCK)
	; - falling XCK edge
	ldi		r16, (1<<RXEN)  
	out		UCSRB, r16
	ldi		r16, (1<<UMSEL) + (1<<UPM1)  + (1<<UPM0) + (1<<UCSZ1)  + (1<<UCSZ0) + (0<<UCPOL)    
	out		UCSRC, r16
main:
	; ur se aktivovan sloupec
	in		XL, PIND
	rol		XL
	swap	XL
	andi	XL, 15
	; nate se stav klves danho sloupce
	ld		r16, X
	; nakonfiguruje se vstup (bu vstup+pull-up nebo log. 0)
	out		DDRB, r16
	eor		r16, _0x1f
	out		PORTB, r16
	; byl pijat znak z klvesnice?
	sbis	UCSRA, RXC
	rjmp	main
	; pro jistotu se tvme jako nestlaen klvesa, dokud nezpracujeme nov pijat znak
	out		PORTB, _0x1f
	out		DDRB, _0x00
	in		ZL, UDR
	; uvolnn klvesy?
	cpi		ZL, 0xF0
	brne    ext
	; nastavme jen pznak T
	set
	rjmp	main
ext:
	cpi		ZL, 0xE0
	; extended?
	brne	press
	; pouij se jin tabulky
	ldi		ZH, HIGH(2*exttable1)
	rjmp	main
	; z klvesnice piel znak, dochz k jeho zpracovn	
press:
	lpm		XL, Z
	; shift?
	cpi		XL, 0xFF
	breq	shift
	; stop?
	cpi		XL, 0xFE
	breq	stop
	inc		ZH
	lpm		r16,Z
	ldi		ZH, HIGH(2*table1)
	ld		r17, X
	brts	release
	or		r16,r17
	st		X, r16
	rjmp	main
release:
	com		r16
	and		r16, r17
	st		X,r16
	clt
	rjmp	main
; emulace stisku/uvolnn shift klvesy
shift:
	ldi		ZH, HIGH(2*table1)
	brts	shrelease
	cbi		PORTA, 0
	sbi		DDRA, 0
	rjmp	main
shrelease:
	cbi		DDRA, 0
	sbi		PORTA, 0
	clt
	rjmp	main
; emulace stisku/uvolnn stop klvesy
stop:
	ldi		ZH, HIGH(2*table1)
	brts	strelease
	cpi		ZL, 0x76		; ESC?
	brne	stop1
	sbis	PORTA, 0		; + SHIFT?
	rjmp	PMDreset		; = RESET
stop1:
	cbi		PORTA, 1
	sbi		DDRA, 1
	rjmp	main
strelease:
	cbi		DDRA, 1
	sbi		PORTA, 1
	clt
	rjmp	main
; emulace stisku/uvolnn kombinace shift+rst = RESET
PMDreset:
	sbi		DDRD, PD1
	ldi		r16,0
pr:	dec		r16
	nop
	brne	pr
	; resetneme i sami sebe
	rjmp	reset
		
; zarovnn na celou strnku
_align:		.org	(_align + 255) & 0xFF00

; Princip pevodu:
;  - table1[Scan Code] = piazen PS/2 klvesy sloupci na PMD klvesnici
;  - table2[Scan Code] = maska bitu, odpovdajc dku klvese na PMD klvesnici
; Nap. A = 1C
;       table1[1C] = 0x00 - prvn sloupec
;       table2[1C] = 0x08 - 4. bit = ve 4. sloupci
; 
; Nap. -> = E0 6B (ipka doleva - extended kd)
;       exttable1[6B] = 0x0C - tinct sloupec
;       exttable2[6B] = 0x04 - 3. bit = ve 3. sloupci
;
; pevodn tabulka 1
; indexy 0-15 odpovdajc sloupci, ve kterm se nachz stisknut klvesa (0-F)
; 0xFF/0xFE - vyhrazeni pro shift/stop
			;  		 0     1     2     3     4     5     6     7     8     9     A     B     C     D     E     F
table1:		.db		0x00, 0x08, 0x00, 0x04, 0x02, 0x00, 0x01, 0x0B, 0x00, 0x09, 0x07, 0x05, 0x03, 0x0D, 0x0C, 0x00  ; 0x00
			.db		0x00, 0x00, 0xFF, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00  ; 0x10
			.db		0x00, 0x03, 0x02, 0x02, 0x02, 0x03, 0x02, 0x00, 0x00, 0x00, 0x04, 0x03, 0x04, 0x03, 0x04, 0x00  ; 0x20
			.db		0x00, 0x06, 0x05, 0x05, 0x04, 0x05, 0x05, 0x00, 0x00, 0x00, 0x07, 0x06, 0x06, 0x06, 0x07, 0x00  ; 0x30
			.db		0x00, 0x08, 0x07, 0x07, 0x08, 0x09, 0x08, 0x00, 0x00, 0x09, 0x0A, 0x08, 0x09, 0x09, 0x0A, 0x00  ; 0x40
			.db		0x00, 0x00, 0x0A, 0x00, 0x0A, 0x0B, 0x00, 0x00, 0x00, 0xFF, 0x0D, 0x0B, 0x00, 0x0B, 0x00, 0x00  ; 0x50
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x03, 0x06, 0x00, 0x00, 0x00  ; 0x60
			.db		0x09, 0x09, 0x01, 0x04, 0x05, 0x07, 0xFE, 0x00, 0x0A, 0x00, 0x02, 0x00, 0x00, 0x08, 0x00, 0x00  ; 0x70
			.db		0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x80
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x90
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xA0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xB0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xC0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xD0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xE0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xF0
; pevodn tabulka 2
; bit odpovdajc dku (1, 2, 4, 8, 0x10)
table2:		.db		0x00, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00	; 0x00
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x02, 0x00, 0x00, 0x00, 0x10, 0x08, 0x08, 0x04, 0x02, 0x00  ; 0x10
			.db		0x00, 0x10, 0x10, 0x08, 0x04, 0x02, 0x02, 0x00, 0x00, 0x10, 0x10, 0x08, 0x04, 0x04, 0x02, 0x00  ; 0x20
			.db		0x00, 0x10, 0x10, 0x08, 0x08, 0x04, 0x02, 0x00, 0x00, 0x00, 0x10, 0x08, 0x04, 0x02, 0x02, 0x00  ; 0x30
			.db		0x00, 0x10, 0x08, 0x04, 0x04, 0x02, 0x02, 0x00, 0x00, 0x10, 0x10, 0x08, 0x08, 0x04, 0x02, 0x00  ; 0x40
			.db		0x00, 0x00, 0x08, 0x00, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00, 0x10, 0x04, 0x00, 0x08, 0x00, 0x00  ; 0x50
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x02, 0x00, 0x02, 0x02, 0x00, 0x00, 0x00  ; 0x60
			.db		0x02, 0x10, 0x02, 0x02, 0x02, 0x02, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x00, 0x02, 0x00, 0x00  ; 0x70
			.db		0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x80
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x90
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xA0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xB0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xC0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xD0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xE0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xF0
; pevodn tabulka pro extended kdy
; indexy 0-15 odpovdajc sloupci, ve kterm se nachz stisknut klvesa (0-F)
					; 0    1     2     3     4     5     6     7     8     9     A     B     C     D     E     F
exttable1:
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x00
			.db		0x00, 0x00, 0x00, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x10
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x20
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x30
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x40
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x50
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0D, 0x00, 0x0C, 0x0D, 0x00, 0x00, 0x00  ; 0x60
			.db		0x0C, 0x0D, 0x0E, 0x00, 0x0E, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x0E, 0x00, 0x00  ; 0x70
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x80
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x90
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xA0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xB0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xC0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xD0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xE0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xF0
exttable2:
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x00
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x10
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x20
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x30
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x40
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x50
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x04, 0x04, 0x00, 0x00, 0x00  ; 0x60
			.db		0x02, 0x02, 0x08, 0x00, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x01, 0x00, 0x00  ; 0x70
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x80
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0x90
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xA0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xB0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xC0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xD0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xE0
			.db		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  ; 0xF0
